'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Symbol = require('babel-runtime/core-js/symbol')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _ast = require('./ast');

var _ast2 = _interopRequireDefault(_ast);

var _symbols = require('./symbols');

var _errors = require('./errors');

var Errors = _interopRequireWildcard(_errors);

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

var HOST = _Symbol();
var PORT = _Symbol();
var TOKEN = _Symbol();
var SERVICE_ID = _Symbol();
var PROTOCOL = _Symbol();
var URL = _Symbol();

var UUID_CHECK = /^[0-9A-F]{8}-([0-9A-F]{4}-){3}[0-9A-F]{12}$/i;

var Client = (function () {
  function Client(opts) {
    _classCallCheck(this, Client);

    var self = function self() {
      return self.expr.apply(self, arguments);
    };
    self.__proto__ = this.__proto__;
    if (opts === void 0) {
      throw new Errors.ClientError('Client contructor takes 1 parameter');
    }
    if (typeof opts.host !== 'string' || opts.host === '') {
      throw new Errors.ClientError('opts.host must be a non-empty string');
    }
    self[HOST] = opts.host;

    if (opts.port !== void 0 && (typeof opts.port !== 'number' || opts.port - Math.floor(opts.port) !== 0)) {
      throw new Errors.ClientError('opts.port must be undefined or a valid port number');
    }
    self[PORT] = opts.port || 80;

    if (typeof opts.accessToken !== 'string' || !UUID_CHECK.test(opts.accessToken)) {
      throw new Errors.ClientError('opts.accessToken must be a uuid');
    }
    self[TOKEN] = opts.accessToken;

    if (typeof opts.serviceId !== 'string') {
      throw new Errors.ClientError('opts.serviceId must be a string');
    }
    self[SERVICE_ID] = opts.serviceId;

    self[PROTOCOL] = opts.ssl === true ? 'https' : 'http';

    var port = self[PORT] !== 80 ? ':' + self[PORT] : '';
    self[URL] = self[PROTOCOL] + '://' + self[HOST] + port + '/query';

    return self;
  }

  _createClass(Client, [{
    key: 'query',

    //query
    value: function query(_query) {
      var res, type, output;
      return _regeneratorRuntime.async(function query$(context$2$0) {
        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            context$2$0.prev = 0;
            context$2$0.next = 3;
            return _regeneratorRuntime.awrap((0, _nodeFetch2['default'])(this[URL], {
              method: 'POST',
              headers: {
                'x-meepcloud-access-token': this[TOKEN],
                'x-meepcloud-service-id': this[SERVICE_ID],
                'content-type': 'application/json'
              },
              body: JSON.stringify(_query)
            }));

          case 3:
            res = context$2$0.sent;
            type = res.headers.get('content-type');
            output = undefined;
            context$2$0.t0 = type.split(';').shift();
            context$2$0.next = context$2$0.t0 === 'application/json' ? 9 : 13;
            break;

          case 9:
            context$2$0.next = 11;
            return _regeneratorRuntime.awrap(res.json());

          case 11:
            output = context$2$0.sent;
            return context$2$0.abrupt('break', 17);

          case 13:
            context$2$0.next = 15;
            return _regeneratorRuntime.awrap(res.body());

          case 15:
            output = context$2$0.sent;
            return context$2$0.abrupt('break', 17);

          case 17:
            if (!output.error) {
              context$2$0.next = 19;
              break;
            }

            throw new Errors.ServerError(output.error);

          case 19:
            return context$2$0.abrupt('return', output);

          case 22:
            context$2$0.prev = 22;
            context$2$0.t1 = context$2$0['catch'](0);

            if (!(context$2$0.t1 instanceof Errors.ServerError)) {
              context$2$0.next = 28;
              break;
            }

            throw context$2$0.t1;

          case 28:
            throw new Errors.ClientError(context$2$0.t1);

          case 29:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this, [[0, 22]]);
    }
  }, {
    key: 'expr',

    //rdb functions
    value: function expr() {
      var _ast$db;

      var res = (_ast$db = _ast2['default'].db(this[SERVICE_ID])).expr.apply(_ast$db, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'http',
    value: function http() {
      var _ast$db2;

      var res = (_ast$db2 = _ast2['default'].db(this[SERVICE_ID])).http.apply(_ast$db2, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'json',
    value: function json() {
      var _ast$db3;

      var res = (_ast$db3 = _ast2['default'].db(this[SERVICE_ID])).json.apply(_ast$db3, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'error',
    value: function error() {
      var _ast$db4;

      var res = (_ast$db4 = _ast2['default'].db(this[SERVICE_ID])).error.apply(_ast$db4, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'random',
    value: function random() {
      var _ast$db5;

      var res = (_ast$db5 = _ast2['default'].db(this[SERVICE_ID])).random.apply(_ast$db5, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'binary',
    value: function binary() {
      var _ast$db6;

      var res = (_ast$db6 = _ast2['default'].db(this[SERVICE_ID])).binary.apply(_ast$db6, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'table',
    value: function table() {
      var _ast$db7;

      var res = (_ast$db7 = _ast2['default'].db(this[SERVICE_ID])).table.apply(_ast$db7, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'do',
    value: function _do() {
      var _ast$db8;

      var res = (_ast$db8 = _ast2['default'].db(this[SERVICE_ID]))['do'].apply(_ast$db8, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'branch',
    value: function branch() {
      var _ast$db9;

      var res = (_ast$db9 = _ast2['default'].db(this[SERVICE_ID])).branch.apply(_ast$db9, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'eq',
    value: function eq() {
      var _ast$db10;

      var res = (_ast$db10 = _ast2['default'].db(this[SERVICE_ID])).eq.apply(_ast$db10, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'ne',
    value: function ne() {
      var _ast$db11;

      var res = (_ast$db11 = _ast2['default'].db(this[SERVICE_ID])).ne.apply(_ast$db11, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'lt',
    value: function lt() {
      var _ast$db12;

      var res = (_ast$db12 = _ast2['default'].db(this[SERVICE_ID])).lt.apply(_ast$db12, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'le',
    value: function le() {
      var _ast$db13;

      var res = (_ast$db13 = _ast2['default'].db(this[SERVICE_ID])).le.apply(_ast$db13, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'gt',
    value: function gt() {
      var _ast$db14;

      var res = (_ast$db14 = _ast2['default'].db(this[SERVICE_ID])).gt.apply(_ast$db14, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'ge',
    value: function ge() {
      var _ast$db15;

      var res = (_ast$db15 = _ast2['default'].db(this[SERVICE_ID])).ge.apply(_ast$db15, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'or',
    value: function or() {
      var _ast$db16;

      var res = (_ast$db16 = _ast2['default'].db(this[SERVICE_ID])).or.apply(_ast$db16, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'and',
    value: function and() {
      var _ast$db17;

      var res = (_ast$db17 = _ast2['default'].db(this[SERVICE_ID])).and.apply(_ast$db17, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'not',
    value: function not() {
      var _ast$db18;

      var res = (_ast$db18 = _ast2['default'].db(this[SERVICE_ID])).not.apply(_ast$db18, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'add',
    value: function add() {
      var _ast$db19;

      var res = (_ast$db19 = _ast2['default'].db(this[SERVICE_ID])).add.apply(_ast$db19, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'sub',
    value: function sub() {
      var _ast$db20;

      var res = (_ast$db20 = _ast2['default'].db(this[SERVICE_ID])).sub.apply(_ast$db20, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'div',
    value: function div() {
      var _ast$db21;

      var res = (_ast$db21 = _ast2['default'].db(this[SERVICE_ID])).div.apply(_ast$db21, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'mul',
    value: function mul() {
      var _ast$db22;

      var res = (_ast$db22 = _ast2['default'].db(this[SERVICE_ID])).mul.apply(_ast$db22, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'mod',
    value: function mod() {
      var _ast$db23;

      var res = (_ast$db23 = _ast2['default'].db(this[SERVICE_ID])).mod.apply(_ast$db23, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'typeOf',
    value: function typeOf() {
      var _ast$db24;

      var res = (_ast$db24 = _ast2['default'].db(this[SERVICE_ID])).typeOf.apply(_ast$db24, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'literal',
    value: function literal() {
      var _ast$db25;

      var res = (_ast$db25 = _ast2['default'].db(this[SERVICE_ID])).literal.apply(_ast$db25, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'map',
    value: function map() {
      var _ast$db26;

      var res = (_ast$db26 = _ast2['default'].db(this[SERVICE_ID])).map.apply(_ast$db26, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'asc',
    value: function asc() {
      var _ast$db27;

      var res = (_ast$db27 = _ast2['default'].db(this[SERVICE_ID])).asc.apply(_ast$db27, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'desc',
    value: function desc() {
      var _ast$db28;

      var res = (_ast$db28 = _ast2['default'].db(this[SERVICE_ID])).desc.apply(_ast$db28, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'ISO8601',
    value: function ISO8601() {
      var _ast$db29;

      var res = (_ast$db29 = _ast2['default'].db(this[SERVICE_ID])).ISO8601.apply(_ast$db29, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'epochTime',
    value: function epochTime() {
      var _ast$db30;

      var res = (_ast$db30 = _ast2['default'].db(this[SERVICE_ID])).epochTime.apply(_ast$db30, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'now',
    value: function now() {
      var _ast$db31;

      var res = (_ast$db31 = _ast2['default'].db(this[SERVICE_ID])).now.apply(_ast$db31, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'time',
    value: function time() {
      var _ast$db32;

      var res = (_ast$db32 = _ast2['default'].db(this[SERVICE_ID])).time.apply(_ast$db32, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'object',
    value: function object() {
      var _ast$db33;

      var res = (_ast$db33 = _ast2['default'].db(this[SERVICE_ID])).object.apply(_ast$db33, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'args',
    value: function args() {
      var _ast$db34;

      var res = (_ast$db34 = _ast2['default'].db(this[SERVICE_ID])).args.apply(_ast$db34, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'geojson',
    value: function geojson() {
      var _ast$db35;

      var res = (_ast$db35 = _ast2['default'].db(this[SERVICE_ID])).geojson.apply(_ast$db35, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'point',
    value: function point() {
      var _ast$db36;

      var res = (_ast$db36 = _ast2['default'].db(this[SERVICE_ID])).point.apply(_ast$db36, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'line',
    value: function line() {
      var _ast$db37;

      var res = (_ast$db37 = _ast2['default'].db(this[SERVICE_ID])).line.apply(_ast$db37, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'polygon',
    value: function polygon() {
      var _ast$db38;

      var res = (_ast$db38 = _ast2['default'].db(this[SERVICE_ID])).polygon.apply(_ast$db38, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'intersects',
    value: function intersects() {
      var _ast$db39;

      var res = (_ast$db39 = _ast2['default'].db(this[SERVICE_ID])).intersects.apply(_ast$db39, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'distance',
    value: function distance() {
      var _ast$db40;

      var res = (_ast$db40 = _ast2['default'].db(this[SERVICE_ID])).distance.apply(_ast$db40, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'circle',
    value: function circle() {
      var _ast$db41;

      var res = (_ast$db41 = _ast2['default'].db(this[SERVICE_ID])).circle.apply(_ast$db41, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'uuid',
    value: function uuid() {
      var _ast$db42;

      var res = (_ast$db42 = _ast2['default'].db(this[SERVICE_ID])).uuid.apply(_ast$db42, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'range',
    value: function range() {
      var _ast$db43;

      var res = (_ast$db43 = _ast2['default'].db(this[SERVICE_ID])).range.apply(_ast$db43, arguments);
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'accessToken',
    get: function get() {
      return this[TOKEN];
    }
  }, {
    key: 'host',
    get: function get() {
      return this[HOST];
    }
  }, {
    key: 'port',
    get: function get() {
      return this[PORT];
    }
  }, {
    key: 'row',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).row;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'monday',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).monday;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'tuesday',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).tuesday;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'wednesday',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).wednesday;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'thursday',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).thursday;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'friday',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).friday;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'saturday',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).saturday;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'sunday',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).sunday;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'january',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).january;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'february',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).february;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'march',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).march;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'april',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).april;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'may',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).may;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'june',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).june;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'july',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).july;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'august',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).august;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'september',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).september;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'october',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).october;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'november',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).november;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'december',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).december;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'minval',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).minval;
      res[_symbols.SymClient] = this;
      return res;
    }
  }, {
    key: 'maxval',
    get: function get() {
      var res = _ast2['default'].db(this[SERVICE_ID]).maxval;
      res[_symbols.SymClient] = this;
      return res;
    }
  }]);

  return Client;
})();

exports['default'] = Client;
module.exports = exports['default'];
//# sourceMappingURL=client.js.map