'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _get = require('babel-runtime/helpers/get')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _base = require('./base');

var ClientError = (function (_Error) {
  function ClientError(msg) {
    _classCallCheck(this, ClientError);

    _get(Object.getPrototypeOf(ClientError.prototype), 'constructor', this).call(this, msg);
    this.name = this.constructor.name;
    this.msg = msg;
    this.message = msg;
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, this);
    }
  }

  _inherits(ClientError, _Error);

  return ClientError;
})(Error);

exports.ClientError = ClientError;

var RqlDriverError = (function (_Error2) {
  function RqlDriverError(msg) {
    _classCallCheck(this, RqlDriverError);

    _get(Object.getPrototypeOf(RqlDriverError.prototype), 'constructor', this).call(this, msg);
    this.name = this.constructor.name;
    this.msg = msg;
    this.message = msg;
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, this);
    }
  }

  _inherits(RqlDriverError, _Error2);

  return RqlDriverError;
})(Error);

exports.RqlDriverError = RqlDriverError;

var ServerError = (function (_Error3) {
  function ServerError(err) {
    _classCallCheck(this, ServerError);

    _get(Object.getPrototypeOf(ServerError.prototype), 'constructor', this).call(this, err.message);
    this.name = err.name;
    this.message = err.message;
    this.msg = err.message;
  }

  _inherits(ServerError, _Error3);

  return ServerError;
})(Error);

exports.ServerError = ServerError;

var RqlServerError = (function (_Error4) {
  function RqlServerError(msg, term, frames) {
    _classCallCheck(this, RqlServerError);

    _get(Object.getPrototypeOf(RqlServerError.prototype), 'constructor', this).call(this, msg);
    this.name = this.constructor.name;
    this.msg = msg;
    this.frames = frames.slice(0);

    if ((0, _base.exists)(term)) {
      if (msg[msg.length - 1] === '.') {
        this.message = msg.slice(0, msg.length - 1) + ' in:\n' + ReqlQueryPrinter.prototype.printQuery(term) + '\n' + RqlQueryPrinter.prototype.printCarrots(term, frames);
      } else {
        this.message = msg + ' in:\n' + RqlQueryPrinter.prototype.printQuery(term) + '\n' + RqlQueryPrinter.prototype.printCarrots(term, frames);
      }
    } else {
      this.message = '' + msg;
    }

    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, this);
    }
  }

  _inherits(RqlServerError, _Error4);

  return RqlServerError;
})(Error);

exports.RqlServerError = RqlServerError;

var RqlRuntimeError = (function (_RqlServerError) {
  function RqlRuntimeError() {
    _classCallCheck(this, RqlRuntimeError);

    _get(Object.getPrototypeOf(RqlRuntimeError.prototype), 'constructor', this).apply(this, arguments);
  }

  _inherits(RqlRuntimeError, _RqlServerError);

  return RqlRuntimeError;
})(RqlServerError);

var RqlCompileError = (function (_RqlServerError2) {
  function RqlCompileError() {
    _classCallCheck(this, RqlCompileError);

    _get(Object.getPrototypeOf(RqlCompileError.prototype), 'constructor', this).apply(this, arguments);
  }

  _inherits(RqlCompileError, _RqlServerError2);

  return RqlCompileError;
})(RqlServerError);

var RqlClientError = (function (_RqlServerError3) {
  function RqlClientError() {
    _classCallCheck(this, RqlClientError);

    _get(Object.getPrototypeOf(RqlClientError.prototype), 'constructor', this).apply(this, arguments);
  }

  _inherits(RqlClientError, _RqlServerError3);

  return RqlClientError;
})(RqlServerError);

var RqlQueryPrinter = (function () {
  function RqlQueryPrinter() {
    _classCallCheck(this, RqlQueryPrinter);
  }

  _createClass(RqlQueryPrinter, [{
    key: 'printQuery',
    value: function printQuery(term) {
      var res = RqlQueryPrinter.joinTree(RqlQueryPrinter.composeTerm(term));
      return res;
    }
  }, {
    key: 'printCarrots',
    value: function printCarrots(term, frames) {
      var tree = undefined;
      if (frames.length === 0) {
        tree = [RqlQueryPrinter.carrotify(RqlQueryPrinter.composeTerm(term))];
      } else {
        tree = RqlQueryPrinter.composeCarrots(term, frames);
      }
      return RqlQueryPrinter.joinTree(tree).replace(/[^\^]/g, ' ');
    }
  }], [{
    key: 'composeTerm',
    value: function composeTerm(term) {
      var args = [];
      for (var arg in term.args) {
        args.push(RqlQueryPrinter.composeTerm(term.args[arg]));
      }
      var optargs = {};
      for (var key in term.optargs) {
        if (!term.optargs.hasOwnProperty(key)) {
          continue;
        }
        optargs[key] = RqlQueryPrinter.composeTerm(term.optargs[key]);
      }
      return term.compose(args, optargs);
    }
  }, {
    key: 'composeCarrots',
    value: function composeCarrots(term, frames) {
      var frame = frames.shift();
      var args = [];
      for (var i = 0, len = term.args.length; i < len; i++) {
        if (frame === i) {
          args.push(RqlQueryPrinter.composeCarrots(term.args[i], frames));
        } else {
          args.push(RqlQueryPrinter.composeTerm(term.args[i]));
        }
      }
      var optargs = {};
      for (var key in term.optargs) {
        if (!term.optargs.hasOwnProperty(key)) {
          continue;
        }
        if (frame === key) {
          optargs[key] = RqlQueryPrinter.composeCarrots(term.optargs[key], frames);
        } else {
          optargs[key] = RqlQueryPrinter.composeTerm(term.optargs[key]);
        }
      }

      if ((0, _base.exists)(frame)) {
        return term.compose(args, optargs);
      } else {
        return RqlQueryPrinter.carrotify(term.compose(args, optargs));
      }
    }
  }, {
    key: 'carrotify',
    value: function carrotify(tree) {
      return [RqlQueryPrinter.carrotMarker, tree];
    }
  }, {
    key: 'joinTree',
    value: function joinTree(tree) {
      var str = '';
      for (var i = 0, len = tree.length; i < len; i++) {
        if (Array.isArray(tree[i])) {
          if (tree[i].length === 2 && tree[i][0] === RqlQueryPrinter.carrotMarker) {
            str += RqlQueryPrinter.joinTree(tree[i][1]).replace(/./g, '^');
          } else {
            str += RqlQueryPrinter.joinTree(tree[i]);
          }
        } else {
          str += tree[i];
        }
      }
      return str;
    }
  }]);

  return RqlQueryPrinter;
})();

RqlQueryPrinter.prototype.carrotMarker = {};
var printQuery = RqlQueryPrinter.prototype.printQuery;
exports.printQuery = printQuery;
//# sourceMappingURL=errors.js.map